use std::io::{self, Read};

use itertools::Itertools;
use tokio::sync::mpsc;
use uuid::Uuid;

use crate::{
    codec2::{Complex, Fsk},
    config::FelinetConfig,
    felinet::PacketIn as SemiPacketIn,
    util::{append_internet_to_packet_route, print_packet},
};

use self::{dc_bias::RemoveDcBias, demod::Demod};

mod dc_bias;
mod demod;
mod packet;

pub fn decode_forever(
    felinet_send: mpsc::Sender<SemiPacketIn>,
    felinet_config: &Option<FelinetConfig>,
    uuid: &Uuid,
) {
    let stdin = io::stdin();
    let iq = stdin
        .bytes()
        .map(|x| x.expect("Could not read from stdin"))
        .tuples()
        .map(|(a, b)| i16::from_le_bytes([a, b]))
        .tuples()
        .map(|(real, imag)| {
            let real = real as f32;
            let imag = imag as f32;
            Complex { real, imag }
        });

    let iq_no_dc = RemoveDcBias::new(iq);

    let soft_bits = Fsk::new(iq_no_dc);

    let demod = Demod::new(soft_bits);

    demod.demod(|mut p| {
        print_packet(&p);

        if let Some(fc) = felinet_config {
            if append_internet_to_packet_route(&fc.callsign, fc.ssid, &mut p).is_none() {
                return;
            };

            if let Ok(semi) = p.semi_encode() {
                let semi = SemiPacketIn {
                    raw: semi.to_vec(),
                    uuid: (*uuid).into(),
                };

                let _ = felinet_send.blocking_send(semi);
            }
        }
    });
}
